@@ -36,6 +36,7 @@ import (
 	"gitlab.com/gitlab-org/gitlab-runner/log"
 	"gitlab.com/gitlab-org/gitlab-runner/network"
 	"gitlab.com/gitlab-org/gitlab-runner/session"
+	"gitlab.com/gitlab-org/gitlab-runner/steps"
 )
 
 const (
@@ -943,6 +944,7 @@ func (mr *RunCommand) processBuildOnRunner(
 	}
 	build.Session = buildSession
 	build.ArtifactUploader = mr.network.UploadRawArtifacts
+	build.ExecuteStepFn = steps.Execute
 
 	trace.SetDebugModeEnabled(build.IsDebugModeEnabled())
 
@@ -950,34 +952,29 @@ func (mr *RunCommand) processBuildOnRunner(
 	mr.buildsHelper.addBuild(build)
 
 	fields := logrus.Fields{
+		"runner":                runner.Name,
 		"job":                   build.ID,
 		"project":               build.JobInfo.ProjectID,
+		"project_full_path":     build.JobInfo.ProjectFullPath,
+		"namespace_id":          build.JobInfo.NamespaceID,
+		"root_namespace_id":     build.JobInfo.RootNamespaceID,
+		"organization_id":       build.JobInfo.OrganizationID,
+		"gitlab_user_id":        build.JobInfo.UserID,
 		"repo_url":              build.RepoCleanURL(),
 		"time_in_queue_seconds": build.JobInfo.TimeInQueueSeconds,
 		"queue_size":            build.JobInfo.QueueSize,
 		"queue_depth":           build.JobInfo.QueueDepth,
 	}
 
+	if build.JobInfo.ScopedUserID != nil {
+		fields["gitlab_scoped_user_id"] = *build.JobInfo.ScopedUserID
+	}
+
 	mr.log().WithFields(fields).Infoln("Added job to processing list")
 	defer func() {
 		if mr.buildsHelper.removeBuild(build) {
 			mr.log().WithFields(fields).Infoln("Removed job from processing list")
-
-			mr.usageLoggerStore(usage_log.Record{
-				Runner: usage_log.Runner{
-					ID:       runner.ShortDescription(),
-					Name:     runner.Name,
-					SystemID: runner.GetSystemID(),
-				},
-				Job: usage_log.Job{
-					URL:             build.JobURL(),
-					DurationSeconds: build.FinalDuration().Seconds(),
-					Status:          build.CurrentState().String(),
-					FailureReason:   build.FailureReason().String(),
-					StartedAt:       build.StartedAt().UTC(),
-					FinishedAt:      build.FinishedAt().UTC(),
-				},
-			})
+			mr.usageLoggerStore(common.UsageLogRecordFrom(runner, build))
 		}
 	}()
 
@@ -1099,6 +1096,17 @@ func (mr *RunCommand) requestJob(
 	}
 
 	trace.SetFailuresCollector(mr.failuresCollector)
+
+	updateResult := mr.network.UpdateJob(*runner, jobCredentials, common.UpdateJobInfo{
+		ID:    jobCredentials.ID,
+		State: common.Running,
+	})
+
+	if updateResult.State == common.UpdateAbort || updateResult.CancelRequested {
+		trace.Finish()
+		return nil, nil, nil
+	}
+
 	return trace, jobData, nil
 }
 
