@@ -16,8 +16,8 @@ import (
 
 	"github.com/bmatcuk/doublestar/v4"
 	"github.com/docker/cli/opts"
-	"github.com/docker/docker/api/types"
 	"github.com/docker/docker/api/types/container"
+	"github.com/docker/docker/api/types/image"
 	"github.com/docker/docker/api/types/network"
 	"github.com/docker/docker/api/types/system"
 	"github.com/docker/docker/client"
@@ -45,7 +45,6 @@ import (
 	"gitlab.com/gitlab-org/gitlab-runner/helpers/featureflags"
 	"gitlab.com/gitlab-org/gitlab-runner/helpers/limitwriter"
 	"gitlab.com/gitlab-org/gitlab-runner/shells"
-	"gitlab.com/gitlab-org/gitlab-runner/steps"
 )
 
 const (
@@ -53,11 +52,11 @@ const (
 	ExecutorStageRun     common.ExecutorStage = "docker_run"
 	ExecutorStageCleanup common.ExecutorStage = "docker_cleanup"
 
-	ExecutorStageCreatingBuildVolumes     common.ExecutorStage = "docker_creating_build_volumes"
-	ExecutorStageCreatingStepRunnerVolume common.ExecutorStage = "docker_creating_step_runner_volume"
-	ExecutorStageCreatingServices         common.ExecutorStage = "docker_creating_services"
-	ExecutorStageCreatingUserVolumes      common.ExecutorStage = "docker_creating_user_volumes"
-	ExecutorStagePullingImage             common.ExecutorStage = "docker_pulling_image"
+	ExecutorStageBootstrap            common.ExecutorStage = "docker_bootstrap"
+	ExecutorStageCreatingBuildVolumes common.ExecutorStage = "docker_creating_build_volumes"
+	ExecutorStageCreatingServices     common.ExecutorStage = "docker_creating_services"
+	ExecutorStageCreatingUserVolumes  common.ExecutorStage = "docker_creating_user_volumes"
+	ExecutorStagePullingImage         common.ExecutorStage = "docker_pulling_image"
 )
 
 const ServiceLogOutputLimit = 64 * 1024
@@ -91,7 +90,7 @@ type executor struct {
 	temporary        []string // IDs of containers that should be removed
 	buildContainerID string
 
-	services []*types.Container
+	services []*container.Summary
 
 	links []string
 
@@ -249,13 +248,13 @@ func (e *executor) expandAndGetDockerImage(
 	allowedImages []string,
 	dockerOptions common.ImageDockerOptions,
 	imagePullPolicies []common.DockerPullPolicy,
-) (*types.ImageInspect, error) {
+) (*image.InspectResponse, error) {
 	imageName, err := e.expandImageName(imageName, allowedImages)
 	if err != nil {
 		return nil, err
 	}
 
-	dockerOptions.Platform = e.ExpandValue(dockerOptions.Platform)
+	dockerOptions = dockerOptions.Expand(e.Build.GetAllVariables())
 
 	image, err := e.pullManager.GetDockerImage(imageName, dockerOptions, imagePullPolicies)
 	if err != nil {
@@ -265,7 +264,7 @@ func (e *executor) expandAndGetDockerImage(
 	return image, nil
 }
 
-func (e *executor) getHelperImage() (*types.ImageInspect, error) {
+func (e *executor) getHelperImage() (*image.InspectResponse, error) {
 	if imageNameFromConfig := e.ExpandValue(e.Config.Docker.HelperImage); imageNameFromConfig != "" {
 		e.BuildLogger.Debugln(
 			"Pull configured helper_image for predefined container instead of import bundled image",
@@ -297,7 +296,7 @@ func (e *executor) getHelperImage() (*types.ImageInspect, error) {
 	return e.pullManager.GetDockerImage(e.helperImageInfo.String(), common.ImageDockerOptions{}, nil)
 }
 
-func (e *executor) getLocalHelperImage() *types.ImageInspect {
+func (e *executor) getLocalHelperImage() *image.InspectResponse {
 	if e.helperImageInfo.Prebuilt == "" {
 		return nil
 	}
@@ -310,16 +309,17 @@ func (e *executor) getLocalHelperImage() *types.ImageInspect {
 	return image
 }
 
-func (e *executor) getBuildImage() (*types.ImageInspect, error) {
+func (e *executor) getBuildImage() (*image.InspectResponse, error) {
 	imageName, err := e.expandImageName(e.Build.Image.Name, []string{})
 	if err != nil {
 		return nil, err
 	}
 
+	dockerOptions := e.Build.Image.ExecutorOptions.Docker.Expand(e.Build.GetAllVariables())
 	imagePullPolicies := e.Build.Image.PullPolicies
 
 	// Fetch image
-	image, err := e.pullManager.GetDockerImage(imageName, e.Build.Image.ExecutorOptions.Docker, imagePullPolicies)
+	image, err := e.pullManager.GetDockerImage(imageName, dockerOptions, imagePullPolicies)
 	if err != nil {
 		return nil, err
 	}
@@ -327,8 +327,8 @@ func (e *executor) getBuildImage() (*types.ImageInspect, error) {
 	return image, nil
 }
 
-func fakeContainer(id string, names ...string) *types.Container {
-	return &types.Container{ID: id, Names: names}
+func fakeContainer(id string, names ...string) *container.Summary {
+	return &container.Summary{ID: id, Names: names}
 }
 
 func (e *executor) parseDeviceString(deviceString string) (device container.DeviceMapping, err error) {
@@ -336,8 +336,7 @@ func (e *executor) parseDeviceString(deviceString string) (device container.Devi
 	parts := strings.Split(deviceString, ":")
 
 	if len(parts) > 3 {
-		err = fmt.Errorf("too many colons")
-		return
+		return device, fmt.Errorf("too many colons")
 	}
 
 	device.PathOnHost = parts[0]
@@ -358,7 +357,7 @@ func (e *executor) parseDeviceString(deviceString string) (device container.Devi
 		device.CgroupPermissions = "rwm"
 	}
 
-	return
+	return device, err
 }
 
 func (e *executor) bindDevices() (err error) {
@@ -422,7 +421,7 @@ func (e *executor) createService(
 	service, version, image string,
 	definition common.Image,
 	linkNames []string,
-) (*types.Container, error) {
+) (*container.Summary, error) {
 	if service == "" {
 		return nil, common.MakeBuildError("invalid service image name: %s", definition.Name)
 	}
@@ -438,8 +437,10 @@ func (e *executor) createService(
 		serviceName = fmt.Sprintf("%s:%s...", service, version) // service:version
 	}
 
+	dockerOptions := definition.ExecutorOptions.Docker.Expand(e.Build.GetAllVariables())
+
 	e.BuildLogger.Println("Starting service", serviceName)
-	serviceImage, err := e.pullManager.GetDockerImage(image, definition.ExecutorOptions.Docker, definition.PullPolicies)
+	serviceImage, err := e.pullManager.GetDockerImage(image, dockerOptions, definition.PullPolicies)
 	if err != nil {
 		return nil, err
 	}
@@ -486,7 +487,7 @@ func (e *executor) createService(
 	return fakeContainer(resp.ID, containerName), nil
 }
 
-func platformForImage(image *types.ImageInspect, opts common.ImageExecutorOptions) *v1.Platform {
+func platformForImage(image *image.InspectResponse, opts common.ImageExecutorOptions) *v1.Platform {
 	if image == nil || opts.Docker.Platform == "" {
 		return nil
 	}
@@ -522,7 +523,7 @@ func (e *executor) createHostConfigForService(imageIsPrivileged bool, devices []
 			Memory:            e.Config.Docker.GetServiceMemory(),
 			MemorySwap:        e.Config.Docker.GetServiceMemorySwap(),
 			MemoryReservation: e.Config.Docker.GetServiceMemoryReservation(),
-			CgroupParent:      e.Config.Docker.ServiceCgroupParent,
+			CgroupParent:      e.getServiceCgroupParent(),
 			CpusetCpus:        e.Config.Docker.ServiceCPUSetCPUs,
 			CPUShares:         e.Config.Docker.ServiceCPUShares,
 			NanoCPUs:          nanoCPUs,
@@ -579,7 +580,7 @@ func (e *executor) createServiceContainerConfig(
 		config.Cmd = definition.Command
 	}
 	config.Entrypoint = e.overwriteEntrypoint(&definition)
-	config.User = string(definition.ExecutorOptions.Docker.User)
+	config.User = string(definition.ExecutorOptions.Docker.Expand(e.Build.GetAllVariables()).User)
 
 	return config
 }
@@ -719,7 +720,7 @@ func (e *executor) isInPrivilegedImageList(imageDefinition common.Image) bool {
 }
 
 type containerConfigurator interface {
-	ContainerConfig(image *types.ImageInspect) (*container.Config, error)
+	ContainerConfig(image *image.InspectResponse) (*container.Config, error)
 	HostConfig() (*container.HostConfig, error)
 	NetworkConfig(aliases []string) *network.NetworkingConfig
 }
@@ -750,7 +751,7 @@ func newDefaultContainerConfigurator(
 	}
 }
 
-func (c *defaultContainerConfigurator) ContainerConfig(image *types.ImageInspect) (*container.Config, error) {
+func (c *defaultContainerConfigurator) ContainerConfig(image *image.InspectResponse) (*container.Config, error) {
 	hostname := c.e.Config.Docker.Hostname
 	if hostname == "" {
 		hostname = c.e.Build.ProjectUniqueName()
@@ -781,7 +782,7 @@ func (e *executor) createContainer(
 	imageDefinition common.Image,
 	allowedInternalImages []string,
 	cfgTor containerConfigurator,
-) (*types.ContainerJSON, error) {
+) (*container.InspectResponse, error) {
 	if e.volumesManager == nil {
 		return nil, errVolumesManagerUndefined
 	}
@@ -835,23 +836,10 @@ func (e *executor) createContainer(
 	return &inspect, err
 }
 
-// addStepRunnerToPath adds the step-runner's path (stepRunnerBinaryPath) to the specific environment's PATH variable.
-func addStepRunnerToPath(env []string) []string {
-	res := []string{}
-	for _, e := range env {
-		if strings.HasPrefix(e, "PATH=") {
-			res = append(res, e+":"+stepRunnerBinaryPath)
-		} else {
-			res = append(res, e)
-		}
-	}
-	return res
-}
-
 func (e *executor) createContainerConfig(
 	containerType string,
 	imageDefinition common.Image,
-	image *types.ImageInspect,
+	image *image.InspectResponse,
 	hostname string,
 	cmd []string,
 ) (*container.Config, error) {
@@ -867,7 +855,16 @@ func (e *executor) createContainerConfig(
 		AttachStderr: true,
 		OpenStdin:    true,
 		StdinOnce:    true,
-		Env:          e.Build.GetAllVariables().StringList(),
+		Entrypoint:   e.overwriteEntrypoint(&imageDefinition),
+	}
+
+	// We should never really need to set the environment variables on the container,
+	// as these are exported via the abstract shell.
+	//
+	// Adding these variables interferes with steps. Given this situation, when native
+	// steps is enabled, we no longer add the env vars to the container.
+	if !e.Build.UseNativeSteps() {
+		config.Env = e.Build.GetAllVariables().StringList()
 	}
 
 	// user config should only be set in build containers
@@ -879,17 +876,6 @@ func (e *executor) createContainerConfig(
 		}
 	}
 
-	// only allow entrypoint overwriting if steps is not enabled OR this is the helper container.
-	if containerType == predefinedContainerType || !e.Build.UseNativeSteps() {
-		config.Entrypoint = e.overwriteEntrypoint(&imageDefinition)
-	}
-
-	if containerType == buildContainerType && e.Build.UseNativeSteps() {
-		// Set the build container's environment to the build IMAGE's environment with the step-runner binary path
-		// appended to PATH.
-		config.Env = addStepRunnerToPath(image.Config.Env)
-	}
-
 	// setting a container's mac-address changed in API version 1.44
 	if e.serverAPIVersion.LessThan(version1_44) {
 		//nolint:staticcheck
@@ -903,7 +889,7 @@ func (e *executor) getBuildContainerUser(imageDefinition common.Image) (string,
 	// runner config takes precedence
 	user := e.Config.Docker.User
 	if user == "" {
-		user = string(imageDefinition.ExecutorOptions.Docker.User)
+		user = string(imageDefinition.ExecutorOptions.Docker.Expand(e.Build.GetAllVariables()).User)
 	}
 
 	if !e.Config.Docker.IsUserAllowed(user) {
@@ -914,6 +900,22 @@ func (e *executor) getBuildContainerUser(imageDefinition common.Image) (string,
 	return user, nil
 }
 
+// getCgroupParent returns the cgroup parent for build containers
+func (e *executor) getCgroupParent() string {
+	if path := e.Config.GetSlotCgroupPath(e.Build.ExecutorData); path != "" {
+		return path
+	}
+	return e.Config.Docker.CgroupParent
+}
+
+// getServiceCgroupParent returns the cgroup parent for service containers
+func (e *executor) getServiceCgroupParent() string {
+	if path := e.Config.GetServiceSlotCgroupPath(e.Build.ExecutorData); path != "" {
+		return path
+	}
+	return e.Config.Docker.ServiceCgroupParent
+}
+
 func (e *executor) createHostConfig(isBuildContainer, imageIsPrivileged bool) (*container.HostConfig, error) {
 	nanoCPUs, err := e.Config.Docker.GetNanoCPUs()
 	if err != nil {
@@ -942,7 +944,7 @@ func (e *executor) createHostConfig(isBuildContainer, imageIsPrivileged bool) (*
 			Memory:            e.Config.Docker.GetMemory(),
 			MemorySwap:        e.Config.Docker.GetMemorySwap(),
 			MemoryReservation: e.Config.Docker.GetMemoryReservation(),
-			CgroupParent:      e.Config.Docker.CgroupParent,
+			CgroupParent:      e.getCgroupParent(),
 			CpusetCpus:        e.Config.Docker.CPUSetCPUs,
 			CpusetMems:        e.Config.Docker.CPUSetMems,
 			CPUShares:         e.Config.Docker.CPUShares,
@@ -985,15 +987,6 @@ func (e *executor) createHostConfig(isBuildContainer, imageIsPrivileged bool) (*
 func (e *executor) startAndWatchContainer(ctx context.Context, id string, input io.Reader) error {
 	dockerExec := exec.NewDocker(e.Context, e.dockerConn, e.waiter, e.Build.Log())
 
-	// Use stepsDocker exec implementation if steps is enabled and this is the build container.
-	if id == e.buildContainerID && e.Build.UseNativeSteps() {
-		request, err := steps.NewRequest(e.Build)
-		if err != nil {
-			return common.MakeBuildError("creating steps request: %w", err)
-		}
-		dockerExec = exec.NewStepsDocker(e.Context, e.dockerConn, e.waiter, e.Build.Log(), request)
-	}
-
 	stdout := e.BuildLogger.Stream(buildlogger.StreamWorkLevel, buildlogger.Stdout)
 	defer stdout.Close()
 
@@ -1247,7 +1240,7 @@ func (e *executor) createDependencies() error {
 		e.createVolumesManager,
 		e.createVolumes,
 		e.createBuildVolume,
-		e.createStepRunnerVolume,
+		e.bootstrap,
 		e.createServices,
 	}
 
@@ -1322,21 +1315,6 @@ func (e *executor) createBuildVolume() error {
 	return nil
 }
 
-func (e *executor) createStepRunnerVolume() error {
-	if !e.Build.UseNativeSteps() {
-		return nil
-	}
-
-	e.SetCurrentStage(ExecutorStageCreatingStepRunnerVolume)
-	e.BuildLogger.Debugln("Creating step-runner volume...")
-
-	if e.volumesManager == nil {
-		return errVolumesManagerUndefined
-	}
-
-	return e.volumesManager.CreateTemporary(e.Context, stepRunnerBinaryPath)
-}
-
 func (e *executor) Prepare(options common.ExecutorPrepareOptions) error {
 	e.SetCurrentStage(ExecutorStagePrepare)
 
@@ -1599,7 +1577,7 @@ func (e *executor) cleanupVolume(ctx context.Context) error {
 	return nil
 }
 
-func (e *executor) createHostConfigForServiceHealthCheck(service *types.Container) *container.HostConfig {
+func (e *executor) createHostConfigForServiceHealthCheck(service *container.Summary) *container.HostConfig {
 	var legacyLinks []string
 	if e.networkMode.UserDefined() == "" {
 		legacyLinks = append(legacyLinks, service.Names[0]+":service")
@@ -1628,7 +1606,7 @@ func (e *executor) createHostConfigForServiceHealthCheck(service *types.Containe
 // The legacy container links (https://docs.docker.com/network/links/) network
 // feature is deprecated. When we remove support for links, the healthcheck
 // system can be updated to no longer rely on environment variables
-func (e *executor) addServiceHealthCheckEnvironment(service *types.Container) ([]string, error) {
+func (e *executor) addServiceHealthCheckEnvironment(service *container.Summary) ([]string, error) {
 	environment := []string{}
 
 	if e.networkMode.UserDefined() != "" {
@@ -1650,7 +1628,7 @@ func (e *executor) addServiceHealthCheckEnvironment(service *types.Container) ([
 }
 
 //nolint:gocognit
-func (e *executor) getContainerExposedPorts(container *types.Container) ([]int, error) {
+func (e *executor) getContainerExposedPorts(container *container.Summary) ([]int, error) {
 	inspect, err := e.dockerConn.ContainerInspect(e.Context, container.ID)
 	if err != nil {
 		return nil, err
