@@ -13,8 +13,8 @@ import (
 	"testing"
 	"time"
 
-	"github.com/docker/docker/api/types"
 	"github.com/docker/docker/api/types/container"
+	"github.com/docker/docker/api/types/image"
 	"github.com/docker/docker/api/types/network"
 	"github.com/docker/docker/api/types/system"
 	"github.com/docker/go-connections/nat"
@@ -34,7 +34,7 @@ import (
 	"gitlab.com/gitlab-org/gitlab-runner/helpers/trace"
 )
 
-func testServiceFromNamedImage(t *testing.T, description, imageName, serviceName string) {
+func testServiceFromNamedImage(t *testing.T, description, imageName, serviceName, platform string) {
 	c := docker.NewMockClient(t)
 	p := pull.NewMockManager(t)
 
@@ -82,7 +82,7 @@ func testServiceFromNamedImage(t *testing.T, description, imageName, serviceName
 	options := common.ImageDockerOptions{}
 
 	p.On("GetDockerImage", imageName, options, []common.DockerPullPolicy(nil)).
-		Return(&types.ImageInspect{ID: "helper-image"}, nil).
+		Return(&image.InspectResponse{ID: "helper-image"}, nil).
 		Once()
 
 	c.On(
@@ -117,15 +117,30 @@ func testServiceFromNamedImage(t *testing.T, description, imageName, serviceName
 	err = e.createVolumesManager()
 	require.NoError(t, err)
 
-	linksMap := make(map[string]*types.Container)
-	err = e.createFromServiceDefinition(0, common.Image{Name: description}, linksMap)
+	imageConfig := common.Image{
+		Name: description,
+	}
+	if platform != "" {
+		imageConfig.ExecutorOptions = common.ImageExecutorOptions{
+			Docker: common.ImageDockerOptions{
+				Platform: "${PLATFORM}",
+			},
+		}
+		e.Build.Variables = append(e.Build.Variables, common.JobVariable{
+			Key:   "PLATFORM",
+			Value: platform,
+		})
+	}
+
+	linksMap := make(map[string]*container.Summary)
+	err = e.createFromServiceDefinition(0, imageConfig, linksMap)
 	assert.NoError(t, err)
 }
 
 func TestServiceFromNamedImage(t *testing.T) {
 	for _, test := range service_test.Services {
 		t.Run(test.Description, func(t *testing.T) {
-			testServiceFromNamedImage(t, test.Description, test.Image, test.Service)
+			testServiceFromNamedImage(t, test.Description, test.Image, test.Service, test.Platform)
 		})
 	}
 }
@@ -342,7 +357,7 @@ func TestDockerWithDockerConfigAlwaysAndIfNotPresentAndWithServiceImagePullPolic
 	c, e := createExecutorForTestDockerConfiguration(t, dockerConfig, cce)
 
 	c.On("ImageInspectWithRaw", mock.Anything, "alpine:latest").
-		Return(types.ImageInspect{ID: "123"}, []byte{}, nil).Once()
+		Return(image.InspectResponse{ID: "123"}, []byte{}, nil).Once()
 	c.On("NetworkList", mock.Anything, mock.Anything).
 		Return([]network.Summary{}, nil).Once()
 	c.On("ContainerRemove", mock.Anything, mock.Anything, mock.Anything).
@@ -633,7 +648,7 @@ func TestAddServiceHealthCheck(t *testing.T) {
 			networkMode: "test",
 			dockerClientAssertions: func(c *docker.MockClient) {
 				c.On("ContainerInspect", mock.Anything, mock.Anything).
-					Return(types.ContainerJSON{
+					Return(container.InspectResponse{
 						Config: &container.Config{
 							ExposedPorts: nat.PortSet{
 								"1000/tcp": {},
@@ -651,7 +666,7 @@ func TestAddServiceHealthCheck(t *testing.T) {
 			networkMode: "test",
 			dockerClientAssertions: func(c *docker.MockClient) {
 				c.On("ContainerInspect", mock.Anything, mock.Anything).
-					Return(types.ContainerJSON{
+					Return(container.InspectResponse{
 						Config: &container.Config{
 							ExposedPorts: nat.PortSet{
 								"1000/tcp":  {},
@@ -684,7 +699,7 @@ func TestAddServiceHealthCheck(t *testing.T) {
 			networkMode: "test",
 			dockerClientAssertions: func(c *docker.MockClient) {
 				c.On("ContainerInspect", mock.Anything, mock.Anything).
-					Return(types.ContainerJSON{
+					Return(container.InspectResponse{
 						Config: &container.Config{
 							ExposedPorts: nat.PortSet{
 								"1000-1100": {},
@@ -721,7 +736,7 @@ func TestAddServiceHealthCheck(t *testing.T) {
 			networkMode: "test",
 			dockerClientAssertions: func(c *docker.MockClient) {
 				c.On("ContainerInspect", mock.Anything, mock.Anything).
-					Return(types.ContainerJSON{
+					Return(container.InspectResponse{
 						Config: &container.Config{
 							ExposedPorts: nat.PortSet{
 								"1000/tcp": {},
@@ -742,7 +757,7 @@ func TestAddServiceHealthCheck(t *testing.T) {
 			networkMode: "test",
 			dockerClientAssertions: func(c *docker.MockClient) {
 				c.On("ContainerInspect", mock.Anything, mock.Anything).
-					Return(types.ContainerJSON{
+					Return(container.InspectResponse{
 						Config: &container.Config{
 							ExposedPorts: nat.PortSet{
 								"1000/tcp": {},
@@ -763,7 +778,7 @@ func TestAddServiceHealthCheck(t *testing.T) {
 			networkMode: "test",
 			dockerClientAssertions: func(c *docker.MockClient) {
 				c.On("ContainerInspect", mock.Anything, mock.Anything).
-					Return(types.ContainerJSON{
+					Return(container.InspectResponse{
 						Config: &container.Config{
 							ExposedPorts: nat.PortSet{
 								"1000/tcp": {},
@@ -781,7 +796,7 @@ func TestAddServiceHealthCheck(t *testing.T) {
 			networkMode: "test",
 			dockerClientAssertions: func(c *docker.MockClient) {
 				c.On("ContainerInspect", mock.Anything, mock.Anything).
-					Return(types.ContainerJSON{
+					Return(container.InspectResponse{
 						Config: &container.Config{
 							ExposedPorts: nat.PortSet{},
 						},
@@ -794,7 +809,7 @@ func TestAddServiceHealthCheck(t *testing.T) {
 			networkMode: "test",
 			dockerClientAssertions: func(c *docker.MockClient) {
 				c.On("ContainerInspect", mock.Anything, mock.Anything).
-					Return(types.ContainerJSON{}, fmt.Errorf("%v", "test error")).
+					Return(container.InspectResponse{}, fmt.Errorf("%v", "test error")).
 					Once()
 			},
 			expectedErr: fmt.Sprintf("get container exposed ports: %v", "test error"),
@@ -814,7 +829,7 @@ func TestAddServiceHealthCheck(t *testing.T) {
 				dockerConn:  &dockerConnection{Client: client},
 			}
 
-			service := &types.Container{
+			service := &container.Summary{
 				ID:    "0000000000000000000000000000000000000000000000000000000000000000",
 				Names: []string{"default"},
 			}
@@ -905,7 +920,7 @@ func Test_Executor_captureContainerLogs(t *testing.T) {
 }
 
 func Test_Executor_captureContainersLogs(t *testing.T) {
-	containers := []*types.Container{
+	containers := []*container.Summary{
 		{
 			ID:    "000000000000000000000000000000000",
 			Names: []string{"some container"},
@@ -918,7 +933,7 @@ func Test_Executor_captureContainersLogs(t *testing.T) {
 		},
 	}
 
-	linksMap := map[string]*types.Container{
+	linksMap := map[string]*container.Summary{
 		"one":       containers[0],
 		"two":       containers[1],
 		"two-alias": containers[1],
