@@ -44,7 +44,6 @@ const (
 	UseLegacyS3CacheAdapter              string = "FF_USE_LEGACY_S3_CACHE_ADAPTER"
 	GitURLsWithoutTokens                 string = "FF_GIT_URLS_WITHOUT_TOKENS"
 	WaitForPodReachable                  string = "FF_WAIT_FOR_POD_TO_BE_REACHABLE"
-	UseNativeSteps                       string = "FF_USE_NATIVE_STEPS"
 	MaskAllDefaultTokens                 string = "FF_MASK_ALL_DEFAULT_TOKENS"
 	ExportHighCardinalityMetrics         string = "FF_EXPORT_HIGH_CARDINALITY_METRICS"
 	UseFleetingAcquireHeartbeats         string = "FF_USE_FLEETING_ACQUIRE_HEARTBEATS"
@@ -376,14 +375,6 @@ var flags = []FeatureFlag{
 		Deprecated:   false,
 		Description:  "When enabled, the runner waits for the Pod status to be 'Running', and for the Pod to be ready with its certificates attached.",
 	},
-	{
-		Name:         UseNativeSteps,
-		DefaultValue: true,
-		Deprecated:   false,
-		Description: "When enabled and when a job is specified using the 'run' keyword, steps are executed using " +
-			"[Native Step Runner Integration](../executors/docker.md#native-step-runner-integration). " +
-			"This applies only to executors that support native steps integration like Docker, Docker Autoscaler, Docker for Windows.",
-	},
 	{
 		Name:         MaskAllDefaultTokens,
 		DefaultValue: true,
