@@ -160,7 +160,7 @@ func (b *BashWriter) SetupGitCredHelper(confFile, section, user string) {
 			`git config -f %[1]s --replace-all %[2]s %[3]s && `+
 				`git config -f %[1]s --add %[2]s %[4]s && `+
 				`git config -f %[1]s %[5]s %[6]s`,
-			b.escape(confFile),
+			doubleQuote(confFile),
 			helperSection,
 			emptyArg,
 			b.escape(credHelperCommand),
@@ -220,6 +220,10 @@ func (b *BashWriter) Variable(variable common.JobVariable) {
 	}
 }
 
+func (b *BashWriter) ExportRaw(name, value string) {
+	b.Linef(`export %s=%s`, b.escape(name), doubleQuote(value))
+}
+
 func (b *BashWriter) DotEnvVariables(baseFilename string, variables map[string]string) string {
 	dotEnvFile := b.TmpFile(baseFilename)
 
@@ -250,14 +254,19 @@ func (b *BashWriter) IfFile(path string) {
 }
 
 func (b *BashWriter) IfCmd(cmd string, arguments ...string) {
-	cmdline := b.buildCommand(b.escape, cmd, arguments...)
-	b.Linef("if %s >/dev/null 2>&1; then", cmdline)
-	b.Indent()
+	b.ifCmd(b.buildCommand(b.escape, cmd, arguments...) + " >/dev/null 2>&1")
 }
 
 func (b *BashWriter) IfCmdWithOutput(cmd string, arguments ...string) {
-	cmdline := b.buildCommand(b.escape, cmd, arguments...)
-	b.Linef("if %s; then", cmdline)
+	b.ifCmd(b.buildCommand(b.escape, cmd, arguments...))
+}
+
+func (b *BashWriter) IfCmdWithOutputArgExpand(cmd string, arguments ...string) {
+	b.ifCmd(b.buildCommand(doubleQuote, cmd, arguments...))
+}
+
+func (b *BashWriter) ifCmd(cmdline string) {
+	b.Linef("if %s ; then", cmdline)
 	b.Indent()
 }
 
@@ -312,14 +321,20 @@ func (b *BashWriter) RmFile(path string) {
 func (b *BashWriter) RmFilesRecursive(path string, name string) {
 	b.IfDirectory(path)
 	// `find -delete` is not portable; https://unix.stackexchange.com/a/194348
-	b.Linef("find %q -name %q -type f -exec rm -f {} +", path, name)
+	// The '+' `find -exec` terminator is the default as it performs less invocations on supported systems.
+	//   Other systems (e.g. z/OS) will use the ';' terminator when the initial 'test find' command fails, causing `$et` to be updated.
+	//   The intitial `test find` command is used to prevent undesirable stderr output from a failed execution using the `+` terminator.
+	b.Linef(`et='+' ; find /dev/null -exec true {} + 2>/dev/null || et=';' ; find %q -name %q -type f -exec rm -f {} "${et}"`, path, name)
 	b.EndIf()
 }
 
 func (b *BashWriter) RmDirsRecursive(path string, name string) {
 	b.IfDirectory(path)
 	// `find -delete` is not portable; https://unix.stackexchange.com/a/194348
-	b.Linef("find %q -name %q -type d -depth -exec rm -rf -- {} +", path, name)
+	// The '+' `find -exec` terminator is the default as it performs less invocations on supported systems.
+	//   Other systems (e.g. z/OS) will use the ';' terminator when the initial 'test find' command fails, causing `$et` to be updated.
+	//   The intitial `test find` command is used to prevent undesirable stderr output from a failed execution using the `+` terminator.
+	b.Linef(`et='+' ; find /dev/null -exec true {} + 2>/dev/null || et=';' ; find %q -name %q -type d -depth -exec rm -rf -- {} "${et}"`, path, name)
 	b.EndIf()
 }
 
