@@ -293,7 +293,7 @@ func (p *PsWriter) SetupGitCredHelper(confFile, section, user string) {
 	//		- if it's enabled, we need to accept that; we can control exact behavior but we have to use it
 	// We run all that in a script block, so that setting PSNativeCommandArgumentPassing only effects this (and child)
 	// scopes.
-	p.Line(strings.Join([]string{
+	p.lines(
 		`& {`,
 		`$psVer = [Version]$PSVersionTable.PSVersion`,
 		`$needsSpecialArgQuoting = ($psVer -lt [Version]"7.3")`,
@@ -302,18 +302,28 @@ func (p *PsWriter) SetupGitCredHelper(confFile, section, user string) {
 		`  $needsSpecialArgQuoting = $False`,
 		`}`,
 		`if ($needsSpecialArgQuoting) {`,
-		clearCmd + " " + `'""'`,
-		setCmd + " " + psSingleQuote(strings.ReplaceAll(credHelperCommand, `"`, `\"`)),
+		clearCmd+" "+`'""'`,
+		setCmd+" "+psSingleQuote(strings.ReplaceAll(credHelperCommand, `"`, `\"`)),
 		`} else {`,
-		clearCmd + " " + `''`,
-		setCmd + " " + psSingleQuote(credHelperCommand),
+		clearCmd+" "+`''`,
+		setCmd+" "+psSingleQuote(credHelperCommand),
 		`}`,
 		userCmd,
 		`}`,
-	}, p.EOL))
+	)
 	p.CheckForErrors()
 }
 
+func (p *PsWriter) lines(texts ...string) {
+	lines := []string{}
+
+	for _, t := range texts {
+		lines = append(lines, strings.FieldsFunc(t, func(r rune) bool { return r == '\n' })...)
+	}
+
+	p.Line(strings.Join(lines, p.EOL))
+}
+
 func (p *PsWriter) CommandArgExpand(command string, arguments ...string) {
 	p.Line(p.buildCommand(psDoubleQuote, command, arguments...))
 	p.checkErrorLevel()
@@ -404,6 +414,12 @@ func (p *PsWriter) Variable(variable common.JobVariable) {
 	p.Linef("$env:%s=$%s", variable.Key, variable.Key)
 }
 
+func (p *PsWriter) ExportRaw(name, value string) {
+	quotedVal := psDoubleQuote(value)
+	p.Linef("$%s=%s", name, quotedVal)
+	p.Linef("$env:%s=%s", name, quotedVal)
+}
+
 func (p *PsWriter) DotEnvVariables(baseFilename string, variables map[string]string) string {
 	p.MkDir(p.TemporaryPath)
 	dotEnvFile := p.TmpFile(baseFilename)
@@ -442,6 +458,10 @@ func (p *PsWriter) IfCmdWithOutput(cmd string, arguments ...string) {
 	p.ifInTryCatch(p.buildCommand(psSingleQuote, cmd, arguments...))
 }
 
+func (p *PsWriter) IfCmdWithOutputArgExpand(cmd string, arguments ...string) {
+	p.ifInTryCatch(p.buildCommand(psDoubleQuote, cmd, arguments...))
+}
+
 func (p *PsWriter) IfGitVersionIsAtLeast(version string) {
 	p.Line(`if ([Version]((git version | Select-String "git version (\d+(?:\.\d+)*)").Matches.Groups[1].Value) -ge [Version]"` + version + `") {`)
 	p.Indent()
@@ -713,14 +733,18 @@ func (b *PowerShell) scriptArgs(script *common.ShellConfiguration) []string {
 }
 
 func (b *PowerShell) passAsFile(info common.ShellScriptInfo) bool {
+	// custom executor always passes script by a file
+	if info.Build.Runner.Executor == "custom" {
+		return true
+	}
+
 	// if DisablePowershellStdin is false, powershell is passed via stdin
 	if !info.Build.IsFeatureFlagOn(featureflags.DisablePowershellStdin) {
 		return false
 	}
 
-	// we only support powershell script by a file for shell & custom executors
-	switch info.Build.Runner.Executor {
-	case "shell", "custom":
+	// we only support powershell script by a file for shell
+	if info.Build.Runner.Executor == "shell" {
 		return true
 	}
 
